// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "Program.h"
#include <qmsgbox.h>
#include <stdio.h>


// ***************
//  Here, we define the actual functions



//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class Program
//
//||||||||||||||||||||||||||||||||||||||||||||||||





// Constructor for the Program object.
//

Program::Program( AudioDev *devname ):
   APDialog( NULL, NULL, TRUE, STYLEFLAGS )
{
   QFont dffont( "helvetica", 10 );

   tracks = new QListBox( this, "tracks" );
   tracks->setGeometry( 20, 40, 100, 270 );
   tracks->setFont( dffont );
   connect( tracks, SIGNAL(selected(int)), SLOT(addProgItem(int)) );
   connect( tracks, SIGNAL(highlighted(int)), SLOT(clearMove(int)) );
   tracks->setFrameStyle( 50 );
   tracks->setLineWidth( 2 );
   tracks->setAutoUpdate(true);
   
   newprog = new QListBox( this, "newprog" );
   newprog->setGeometry( 230, 40, 100, 270 );
   newprog->setFont( dffont );
   newprog->setFrameStyle( 50 );
   newprog->setLineWidth( 2 );
   newprog->setAutoUpdate(true);
   newprog->insertItem(" ");
   connect( newprog, SIGNAL(highlighted(int)), SLOT(endMove(int)) );

   resize( 350, 400 );
   createChildren(dffont);
   dev = devname;
   isMoving = false;
   moveThis = -1;
}


// Destructor for the Program object.
//
Program::~Program( void )
{
   delete tracks;
   delete newprog;
}


// Slot to open the dialog and start the local execution loop
//
void Program::open()
{
   int status,i;
   char buf[MAXCHARLEN];

   // Make sure we have something to change...
   if(dev->ToC == NULL)
	 {
	 QMessageBox::message("Error!",
					  "Nothing to program!",
					  "close");
	 return;
	 }

   // Okay - there's something there.  Let's set up the track list
   // box.
   tracks->clear();
   for(i=0; i<dev->ToC->len(); i++)
	 {
	 sprintf(buf,"%d :: %.248s",i+1,dev->ToC->name(i));
	 tracks->insertItem(buf);
	 }

   // Enter the control loop.
   status = exec();
   if(status == Accepted)
	 startProg();
   else
	 clearProg();
}



//............................................................
//
// Protected Methods
//
//............................................................



// Creates and places the child objects
//
void Program::createChildren(QFont& dffont)
{
   // Start
   QPushButton* tmpQPushButton;
   tmpQPushButton = new QPushButton( this, "startb" );
   tmpQPushButton->setGeometry( 10, 360, 50, 30 );
   connect( tmpQPushButton, SIGNAL(clicked()), SLOT(accept()) );
   tmpQPushButton->setText( "Start" );
   tmpQPushButton->setAutoRepeat( FALSE );
   tmpQPushButton->setAutoResize( FALSE );

   // Cancel
   tmpQPushButton = new QPushButton( this, "cancelb" );
   tmpQPushButton->setGeometry( 150, 360, 50, 30 );
   connect( tmpQPushButton, SIGNAL(clicked()), SLOT(reject()) );
   tmpQPushButton->setText( "Cancel" );
   tmpQPushButton->setAutoRepeat( FALSE );
   tmpQPushButton->setAutoResize( FALSE );
   
   // Add
   tmpQPushButton = new QPushButton( this, "addb" );
   tmpQPushButton->setGeometry( 130, 50, 90, 30 );
   connect( tmpQPushButton, SIGNAL(clicked()), SLOT(addProgItem()) );
   tmpQPushButton->setText( "->  Add  ->" );
   tmpQPushButton->setAutoRepeat( FALSE );
   tmpQPushButton->setAutoResize( FALSE );
   
   // Delete
   tmpQPushButton = new QPushButton( this, "delb" );
   tmpQPushButton->setGeometry( 130, 90, 90, 30 );
   connect( tmpQPushButton, SIGNAL(clicked()), SLOT(delProgItem()) );
   tmpQPushButton->setText( "|<- Delete <-" );
   tmpQPushButton->setAutoRepeat( FALSE );
   tmpQPushButton->setAutoResize( FALSE );
   
   // Move
   moveb = new QPushButton( this, "moveb" );
   moveb->setGeometry( 130, 130, 90, 30 );
   connect( moveb, SIGNAL(clicked()), SLOT(moveProgItem()) );
   moveb->setText( "Move" );
   moveb->setAutoRepeat( FALSE );
   moveb->setAutoResize( FALSE );
   
   // Move to End
   tmpQPushButton = new QPushButton( this, "movetoendb" );
   tmpQPushButton->setGeometry( 130, 170, 90, 30 );
   connect( tmpQPushButton, SIGNAL(clicked()), SLOT(moveToEnd()) );
   tmpQPushButton->setText( "Move to End" );
   tmpQPushButton->setAutoRepeat( FALSE );
   tmpQPushButton->setAutoResize( FALSE );
   
   // Clear
   tmpQPushButton = new QPushButton( this, "clearb" );
   tmpQPushButton->setGeometry( 130, 270, 90, 30 );
   connect( tmpQPushButton, SIGNAL(clicked()), SLOT(clearProg()) );
   tmpQPushButton->setText( "Clear Program" );
   tmpQPushButton->setAutoRepeat( FALSE );
   tmpQPushButton->setAutoResize( FALSE );


   // The two labels
   QLabel* tmpQLabel;
   tmpQLabel = new QLabel( this, "track_l" );
   tmpQLabel->setGeometry( 20, 20, 40, 15 );
   tmpQLabel->setFont( dffont );
   tmpQLabel->setText( "Track #:" );
   tmpQLabel->setAlignment( AlignLeft | AlignVCenter );
   tmpQLabel->setBuddy( tracks );
   
   tmpQLabel = new QLabel( this, "prog_l" );
   tmpQLabel->setGeometry( 230, 20, 50, 15 );
   tmpQLabel->setFont( dffont );
   tmpQLabel->setText( "Program:" );
   tmpQLabel->setAlignment( AlignLeft | AlignVCenter );
   tmpQLabel->setBuddy( newprog );
}


// Creates the new program and emits the start signal.  Does not clear
// the program list.
//
void Program::startProg(void)
{
   int nulen,i;

   nulen = newprog->count() - 1;
   if(nulen == 0)
	 {
	 dev->ToC->resetPlaylist();
	 return;
	 }

   dev->ToC->pLen = nulen;
   if(dev->ToC->playlist != NULL)
	 delete [] dev->ToC->playlist;

   dev->ToC->playlist = new int[nulen];
   for(i=0; i<nulen; i++)
	 {
	 sscanf(newprog->text(i),"%d",&(dev->ToC->playlist[i]));
	 dev->ToC->playlist[i]--;
	 }
   
   dev->newProgram();
}


// Overloaded protected slot to add the higlighted item in the track list.
//
void Program::addProgItem(void)
{
   int ci;
   
   ci = tracks->currentItem();
   if(ci < 0)
	{
	clearMove(0);
	return;
	}
   else
	 addProgItem(ci);
}


// Adds the requested item, idx1, from the track list to the program.
// It will insert the item into the program at the currently selected
// position in the program list.
// 
void Program::addProgItem(int idx1)
{
   int cipl;

   clearMove(0);
   cipl = newprog->currentItem();
   if(cipl < 0)
	 cipl = newprog->count() - 1;
   newprog->insertItem(tracks->text(idx1),cipl);
}


// Deletes the currently highlighted item from the program list.
// 
void Program::delProgItem(void)
{
   int ci,ep;
   
   clearMove(0);
   ci = newprog->currentItem();
   ep = newprog->count() - 1;
   if( (ci < 0) || (ci == ep) )
	 return;
   else
	 newprog->removeItem(ci);

   newprog->setCurrentItem(ci);
}


// Overloaded protected slot to move a program item
//
void Program::moveProgItem(void)
{
   int ep;
   
   if(isMoving)
	 {
	 clearMove(0);
	 return;
	 }
   else
	 {
	 moveThis = newprog->currentItem();
	 ep = newprog->count() - 1;
	 if( (moveThis < 0) || (moveThis == ep) )
	    {
	    moveThis=-1;
	    return;
	    }
	 isMoving = true;
	 moveb->setText("Move to:");
	 moveb->repaint();
	 return;
	 }
}


// Moves an item in the program list.
// 
void Program::endMove(int nupos)
{
   if(isMoving)
	 {
	 newprog->insertItem(newprog->text(moveThis),nupos);
	 newprog->removeItem(moveThis);
	 clearMove(0);
	 }
   else
	 {
	 return;
	 }   
}


// Terminates a move-in-progress.
//
void Program::clearMove(int i)
{
   isMoving = false;
   moveThis = -1;
   moveb->setText("Move");
   moveb->repaint();
}


// Moves an item to the end of the program list.
// 
void Program::moveToEnd(void)
{
   int ci,ep;
   
   if(isMoving)
	 ci = moveThis;
   else
	 ci = newprog->currentItem();

   ep = newprog->count() - 1;
   if( (ci < 0) || (ci == ep) )
	 {
	 if(isMoving)
	    clearMove(0);
	 return;
	 }

   newprog->insertItem(newprog->text(ci),ep);
   newprog->removeItem(ci);
   newprog->setCurrentItem(ci);
}


// Clears the program
//
void Program::clearProg()
{
   clearMove(0);
   newprog->clear();
   newprog->insertItem(" ");
   newprog->repaint();
}


//||||||||||||||||||||||||||||||||
//
// End
//
