// Contains class definitions
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
#include "pSpecial.h"



//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//
// Button pixmap defines.
//
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// An Up arrow
//
RawPixData uparrowd={ 
  FALSE,
  1.0,
  1.0,
  100,
  3,
  TRUE,
  {".5,.25  .5,.75", ".5,.25  .25,.5", ".5,.25  .75,.5",
   "","","","","","",""},
  "2 2 2",
  "lll" };


// A Down arrow
//
RawPixData downarrowd={ 
  FALSE,
  1.0,
  1.0,
  100,
  3,
  TRUE,
  {".5,.25  .5,.75", ".5,.75  .25,.5", ".5,.75  .75,.5",
   "","","","","","",""},
  "2 2 2",
  "lll" };





// ***************
//  Here, we define the actual functions


//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  Class pSpecial
//
//||||||||||||||||||||||||||||||||||||||||||||||||




// Constructor for the Feature buttons
//
pSpecial::pSpecial( QWidget *parent, bool compact ) : QWidget( parent )
{
   QFont deffnt;
   ftbw_p = QPoint( 0, 0 );
   
   createChildren( compact );
   changeFont( deffnt );
   createMenus();
   moveAll( ftbw_p );
}


// Destructor for the Feature buttons
//
pSpecial::~pSpecial( void )
{
   delete tmodeb;
   delete pmodeb;
   delete playlstb;
   delete progb;
   delete configb;
   delete vol_dnb;
   delete vol_upb;
   delete voltxt;
}


// Creates the child widgets and connects some signals and slots.
//
void pSpecial::createChildren( bool compact )
{
  // Create the buttons
  tmodeb = new QPushButton( this );
  pmodeb = new QPushButton( this );
  playlstb = new QPushButton( this );
  progb = new QPushButton( this );
  configb = new QPushButton( this );
  vol_dnb = new PixButton( this, downarrowd );
  vol_upb = new PixButton( this, uparrowd );
  voltxt = new QLabel( this );
  voltxt->setAlignment( AlignVCenter | AlignRight );

  // Set the scales and default button text.
  hscale = 1.5;
  pmodeb->setText(LABEL_PLAYMODE);
  if(compact)
	{
	wscale = 3.75;

	voltxt->setText(SHORTLABEL_VOLUME);
	tmodeb->setText(SHORTLABEL_TIMEMODE);
	playlstb->setText(SHORTLABEL_PLAYLIST);
	progb->setText(SHORTLABEL_PROGRAM);
	configb->setText(SHORTLABEL_CONFIGURE);
	}
  else
	{
	wscale = 5.67;

	voltxt->setText(LABEL_VOLUME);
	tmodeb->setText(LABEL_TIMEMODE);
	playlstb->setText(LABEL_PLAYLIST);
	progb->setText(LABEL_PROGRAM);
	configb->setText(LABEL_CONFIGURE);
	}
}


// Resizes the children and the main widget.
//
void pSpecial::doRescale(QFont& f)
{
   int wd,ht;
   QLabel tmplabel;
   
   tmplabel.setFont(f);
   tmplabel.setText(EM_STRING);
   tmplabel.resize( tmplabel.sizeHint() );
   ht = (int)( hscale*tmplabel.height() );
   wd = (int)( wscale*tmplabel.width() );

   tmodeb->resize( wd, ht );
   pmodeb->resize( wd, ht );
   playlstb->resize( wd, ht );
   progb->resize( wd, ht );
   configb->resize( wd, ht );
   voltxt->resize( (voltxt->sizeHint()).width(), ht );

   // A bRescale automatically resizes the buttons.
   vol_upb->bRescale( (float)ht/(float)uparrowd.scale );
   vol_dnb->bRescale( (float)ht/(float)downarrowd.scale );
}


// Initializes the positions and sizes of the buttons
//
void pSpecial::locateChildren( void )
{
  tm_p = QPoint( bsep, bsep );
  pm_p = tm_p + QPoint( tmodeb->width() + bsep, 0 );
  plst_p = pm_p + QPoint( pmodeb->width() + bsep, 0 );
  prg_p = plst_p + QPoint( pmodeb->width() + bsep, 0 );
  cfg_p = prg_p + QPoint( progb->width() + bsep, 0 );
  vtxt_p = cfg_p + QPoint( configb->width() + 4*bsep, 0 );
  vd_p = vtxt_p + QPoint( voltxt->width() + bsep, 0 );
  vu_p = vd_p + QPoint( vol_dnb->width() + bsep, 0 );

  ftbw_s = QSize( vu_p.x() + vol_upb->width() + bsep, 
			   tmodeb->height() + 3*bsep );

  resize( ftbw_s );
}


// Moves all of the children en-masse.
//
void pSpecial::moveChildren( void )
{
  tmodeb->move( ftbw_p + tm_p );
  pmodeb->move( ftbw_p + pm_p );
  playlstb->move( ftbw_p + plst_p );
  progb->move( ftbw_p + prg_p );
  configb->move( ftbw_p + cfg_p );
  vol_dnb->move( ftbw_p + vd_p );
  vol_upb->move( ftbw_p + vu_p );
  voltxt->move( ftbw_p + vtxt_p );
}


// Changes the palette
//
void pSpecial::changePalette(QPalette& p)
{
   setPalette(p);
   tmodeb->setPalette(p);
   pmodeb->setPalette(p);
   playlstb->setPalette(p);
   progb->setPalette(p);
   configb->setPalette(p);
   voltxt->setPalette(p);
   tmenu.setPalette(p);
   pmenu.setPalette(p);	 

   vol_upb->setPalette( p );
   vol_dnb->setPalette( p );

   vol_upb->bRepaint();
   vol_dnb->bRepaint();
}


// Changes the font and resizes.
//
void pSpecial::changeFont(QFont& f)
{
   setFont(f);
   tmodeb->setFont(f);
   pmodeb->setFont(f);
   playlstb->setFont(f);
   progb->setFont(f);
   configb->setFont(f);
   voltxt->setFont(f);
   tmenu.setFont(f);
   pmenu.setFont(f);	 

   // Now rescale everything, relocate, and rearrange
   doRescale(f);
   locateChildren();
   moveChildren();
}


// Creates the two popup menus for display and play modes.
//
void pSpecial::createMenus( void )
{
  // Create the items
  tme_id = tmenu.insertItem( LABEL_EL, this, SLOT(tm_e()) );
  tmr_id = tmenu.insertItem( LABEL_REM, this, SLOT(tm_r()) );
  tmea_id = tmenu.insertItem( LABEL_ELALL, this,
						SLOT(tm_ea()) );
  tmra_id = tmenu.insertItem( LABEL_REMALL, this,
						SLOT(tm_ra()) );

  pmn_id = pmenu.insertItem( LABEL_NORM, this, SLOT(pm_n()) );
  pmrpa_id = pmenu.insertItem( LABEL_REPALL, this, SLOT(pm_rpa()) );
  pmrpo_id = pmenu.insertItem( LABEL_REP1, this, SLOT(pm_rpo()) );
  pms_id = pmenu.insertItem( LABEL_SHUFFLE, this, SLOT(pm_s()) );

  // Connect the button signals and menu-open slots.
  connect(tmodeb, SIGNAL(clicked()), SLOT(tmenuOpen()) );
  connect(pmodeb, SIGNAL(clicked()), SLOT(pmenuOpen()) );
}


// Slot to open the display-mode popup menu.
//
void pSpecial::tmenuOpen(void)
{
  tmenuOpen( mapToGlobal( tm_p ) );
}


// Overloaded version.  Opens the menu at the specified position.
//
void pSpecial::tmenuOpen(QPoint ogposn)
{
  tmenu.popup( ogposn );
}


// Slot to open the play-mode popup menu.
//
void pSpecial::pmenuOpen(void)
{
  pmenuOpen( mapToGlobal( pm_p ) );
}


// Overloaded version.  Opens the menu at the specified position.
//
void pSpecial::pmenuOpen(QPoint ogposn)
{
  pmenu.popup( ogposn );
}


// Reimplemented virtual function to receive a mouse pressed
// event. Sends out the pressed() signal.
//
void pSpecial::mousePressEvent( QMouseEvent *me )
{
   emit pressed( mapToGlobal(me->pos()) );
}


// Reimplemented virtual function to receive a mouse move event.  It
// sends out the moving() signal.
//
void pSpecial::mouseMoveEvent( QMouseEvent *me )
{
   emit moving( mapToGlobal(me->pos()) );
}


// These are the slots that handle the display-mode menu.
//
void pSpecial::tm_e(void)
{
  emit newTimeMode( tmElapsed );
}
void pSpecial::tm_r(void)
{
  emit newTimeMode( tmRemaining );
}
void pSpecial::tm_ea(void)
{
  emit newTimeMode( tmElapsed | tmAll );
}
void pSpecial::tm_ra(void)
{
  emit newTimeMode( tmRemaining | tmAll );
}


// These are the slots that handle the play-mode menu.
//
void pSpecial::pm_n(void)
{
  emit newPlayMode( pmNormal );
}
void pSpecial::pm_rpa(void)
{
  emit newPlayMode( pmRepeatAll );
}
void pSpecial::pm_rpo(void)
{
  emit newPlayMode( pmRepeat1 );
}
void pSpecial::pm_s(void)
{
  emit newPlayMode( pmShuffle );
}


//||||||||||||||||||||||||||||||||
//
// End.
//
